searchData={"items":[{"type":"module","title":"Janus","doc":"Authorization superpowers for applications using `Ecto`.\n\nPriorities:\n\n  * Single source of truth - The same rules that authorize loaded data\n    should be able to load authorized data.\n\n  * Authentication-agnostic - Janus should not care about how users\n    are modeled or authenticated.\n\n  * Minimal library footprint - Expose a small but flexible API that\n    can be used to create an optimal authorization interface for each\n    application.\n\n  * Escape hatches where necessary - Complex authorization rules and\n    use-cases should be representable when Janus neglects to provide a\n    short cut.\n\nJanus is split into two primary components:\n\n  * `Janus.Policy` - functions and behaviour for defining _policy\n  modules_, which describe the allowed actors, actions, and resources\n  in your application. This is where you look if you're writing a\n  policy module.\n\n  * `Janus.Authorization` - functions and behaviour used by the rest\n  of your application to authorize and load resources. This is where\n  you look if you're using a policy module.\n\nJanus defines a Mix task to generate the basic policy module that will\nget you started:\n\n    $ mix janus.gen.policy","ref":"Janus.html"},{"type":"module","title":"Installation - Janus","doc":"Janus can be installed by adding `ex_janus` to your deps in `mix.exs`:\n\n    defp deps do\n      [\n        {:ex_janus, \"~> 0.3.3\"}\n      ]\n    end","ref":"Janus.html#module-installation"},{"type":"module","title":"Policies - Janus","doc":"Policy modules are created by invoking `use Janus`, which implements\nboth the `Janus.Policy` and `Janus.Authorization` behaviours:\n\n    defmodule Policy do\n      use Janus\n\n      @impl true\n      def build_policy(policy, _actor) do\n        policy\n      end\n    end\n\nWhen you invoke `use Janus`, default implementations are injected for\nrequired callbacks, except for `c:Janus.Policy.build_policy/2`. This\ncallback is your foundation, as it returns the authorization policy\nfor an individual user of your application.\n\nThe policy above is not very useful (it doesn't allow anyone to do\nanything) but that can be changed by using the `Janus.Policy` API to\ndefine actions, resources, and conditions that make up your\nauthorization rules.\n\n    def build_policy(policy, %User{role: :moderator} = mod) do\n      policy\n      |> allow(Post, :read)\n      |> allow(Post, [:edit, :archive, :unarchive], where: [user: [role: :member]])\n      |> allow(Post, [:edit, :archive, :unarchive], where: [user_id: mod.id])\n      |> deny(Post, :unarchive, where: [archived_by: [role: :admin]])\n    end\n\nSee the `Janus.Policy` documentation for more on defining policies.","ref":"Janus.html#module-policies"},{"type":"module","title":"Authorization - Janus","doc":"With our policy module defined, it can now be used to load and\nauthorize resources.\n\n    iex> Policy.authorize(some_post, :archive, moderator)\n    {:ok, some_post}\n\n    iex> Policy.authorize(post_archived_by_admin, :unarchive, moderator)\n    {:error, :not_authorized}\n\n    iex> Policy.scope(Post, :read, moderator)\n    %Ecto.Query{}\n\n    iex> Policy.scope(Post, :read, moderator) |> Repo.all()\n    [ ... posts the moderator can read ]\n\n    iex> Policy.any_authorized?(Post, :edit, moderator)\n    true # there are rules allowing moderators to edit posts\n\n    iex> Policy.any_authorized?(Post, :delete, moderator)\n    false # there are no rules that allow moderators to delete posts\n\nThese functions make up the `Janus.Authorization` behaviour, and their\ndefinitions were injected by default when we invoked `use Janus`. This\nis the \"public API\" that the rest of your application will use to\nauthorize resources.\n\nSee the `Janus.Authorization` documentation for more.","ref":"Janus.html#module-authorization"},{"type":"module","title":"Integration with `Ecto.Query` - Janus","doc":"The primary assumption that Janus makes is that your resources are\nbacked by an `Ecto.Schema`. Using Ecto's schema reflection\ncapabilities, Janus is able to use the same policy to authorize a\nsingle resource and to construct a composable Ecto query that is aware\nof field types and associations.\n\n    # This query would result in the 5 latest posts that the current\n    # user is authorized to see, preloaded with the user # who made\n    # the post (but only if the current user is  allowed to see that\n    # user).\n\n    Post\n    |> Policy.scope(:read, current_user,\n      preload_authorized: :user\n    )\n    |> order_by(desc: :inserted_at)\n    |> limit(5)\n\nThis integration with Ecto queries is main reason Janus exists.","ref":"Janus.html#module-integration-with-ecto-query"},{"type":"module","title":"Configuration - Janus","doc":"Some defaults can be configured by passing them as options when\ninvoking `use Janus`. Those are:\n\n  * `:repo` - `Ecto.Repo` used to load associations when required by\n    your authorization rules\n\n  * `:load_associations` - Load associations when required by your\n    authorization rules (requires `:repo` config option to be set or\n    to be passed explicitly at the call site), defaults to `false`\n\nFor example:\n\n    defmodule MyApp.Policy do\n        use Janus,\n          repo: MyApp.Repo,\n          load_associations: true\n\n        # ...\n    end\n\nThese defaults will be referenced in the `Janus.Authorization`\ndocumentation where they are used.","ref":"Janus.html#module-configuration"},{"type":"module","title":"Why (not) Janus? - Janus","doc":"Janus was created to scratch an itch: the same rules that authorize\nloaded data should be able to load authorized data. In concrete terms,\na rule that defines whether a user can edit a resource should also be\nable to load all the resources that user can edit.\n\nLoading data this way should be:\n\n  1. efficient - loading everything and then filtering it in-memory\n     doesn't cut it;\n\n  2. composable - it should be possible to add additional conditions\n     when loading data;\n\n  3. ergonomic - authorization should slot-in naturally without major\n     rewrites.\n\nThankfully, integration with `Ecto.Query` solves for all of the above.\nOne only needs authorization rules that can be translated into a\nquery.\n\nAnd thus, Janus was born.","ref":"Janus.html#module-why-not-janus"},{"type":"module","title":"Janus may be a good fit if... - Janus","doc":"* you're authorizing data backed by `Ecto.Schema`. Janus relies on\n    the reflection capabilities of schemas to produce correct queries,\n    cast values, navigate associations, etc.\n\n  * you share interfaces between users with different permissions.\n    Janus allows you to scope queries in a uniform way using the\n    current user (or lack of one), making shared interfaces a natural\n    default.\n\n  * you prefer to have the final say. Janus takes an approach similar\n    to Phoenix, generating code that supports certain conventions\n    while allowing you to override or redefine behavior to fit your\n    preferences.\n\n  * you prefer a functional API for defining rules. Authorization\n    policies are data; adding an authorization rule just transforms\n    that data. Policies can be built using the full extent and natural\n    composability of the Elixir language.","ref":"Janus.html#module-janus-may-be-a-good-fit-if"},{"type":"module","title":"Janus may not be a good fit if... - Janus","doc":"* you're only authorizing actions that don't have an obvious\n    association to data backed by `Ecto.Schema`. For instance, a\n    `:send_welcome_email` action without some kind of `Email` schema.\n    Janus does, however, give you a natural place to define that sort\n    of API yourself (your policy module).\n\n  * you want an easy-to-read DSL for authorization rules. Janus\n    policies are \"just code\", so readability will depend on your own\n    style and structure. If you value readability/scannability very\n    highly, definitely check out [`LetMe`](https://hexdocs.pm/let_me),\n    which provides a great DSL and makes some different trade-offs\n    than Janus does.\n\n  * you want runtime introspection for your authorization rules, like\n    a list of all actions a user can perform. Janus does not currently\n    provide structured access to this information, but you might again\n    turn to [`LetMe`](https://hexdocs.pm/let_me), which provides\n    introspection capabilities.","ref":"Janus.html#module-janus-may-not-be-a-good-fit-if"},{"type":"macro","title":"Janus.__using__/1","doc":"Sets up a module to implement the `Janus.Policy` and\n`Janus.Authorization` behaviours.\n\nUsing `use Janus` does the following:\n\n  * adds the `Janus.Policy` behaviour, imports functions used to\n    define the required callback `c:Janus.Policy.build_policy/2`, and\n    defines a `build_policy/1` helper\n\n  * adds the `Janus.Authorization` behaviour and injects default\n    (overridable) implementations for all callbacks","ref":"Janus.html#__using__/1"},{"type":"macro","title":"Options - Janus.__using__/1","doc":"* `:load_associations` - Load associations when required by your\n    authorization rules (requires `:repo` config option to be set or\n    to be passed explicitly at the call site), defaults to `false`\n\n  * `:repo` - `Ecto.Repo` used to load associations when required by\n    your authorization rules\n\nSee \"Configuration\" section for details.","ref":"Janus.html#__using__/1-options"},{"type":"macro","title":"Example - Janus.__using__/1","doc":"defmodule MyApp.Policy do\n      use Janus, repo: MyApp.Repo\n\n      @impl true\n      def build_policy(policy, _actor) do\n        policy\n        # |> allow(...)\n      end\n    end","ref":"Janus.html#__using__/1-example"},{"type":"type","title":"Janus.action/0","doc":"","ref":"Janus.html#t:action/0"},{"type":"type","title":"Janus.actor/0","doc":"","ref":"Janus.html#t:actor/0"},{"type":"type","title":"Janus.schema_module/0","doc":"","ref":"Janus.html#t:schema_module/0"},{"type":"behaviour","title":"Janus.Authorization","doc":"Authorize and load resources using policies.\n\nPolicy modules expose a minimal API that can be used to authorize and\nload resources throughout the rest of your application.\n\n  * `authorize/4` - authorize an individual, already-loaded resource\n\n  * `scope/4` - construct an `Ecto` query for a schema that will\n    filter results to only those that are authorized\n\n  * `any_authorized?/3` - checks whether the given actor/policy has\n    _any_ access to the given schema for the given action\n\nThese functions will usually be called from your policy module\ndirectly, since wrappers that accept either a policy or an actor are\ninjected when you invoke `use Janus`.  Documentation examples will\nshow usage from your policy module.\n\nSee individual function documentation for details.","ref":"Janus.Authorization.html"},{"type":"function","title":"Janus.Authorization.any_authorized?/3","doc":"Checks whether any permissions are defined for the given schema,\naction, and actor.\n\nThis function is most useful in conjunction with `scope/4`, which\nbuilds an `Ecto` query that filters to only those resources the actor\nis authorized for. If you run the resulting query and receive `[]`, it\nis not possible to determine whether the result is empty because the\nactor wasn't authorized for _any_ resources or because of other\nrestrictions on the query.\n\nFor example, you might use the following pattern to load all the\nresources a user is allowed to read that were inserted in the last day:\n\n    query = from(r in MyResource, where: r.inserted_at > from_now(-1, \"day\"))\n\n    if any_authorized?(query, :read, user) do\n      {:ok, scope(query, :read, user) |> Repo.all()}\n    else\n      {:error, :not_authorized}\n    end\n\nThis would result in `{:ok, results}` if the user is authorized to\nread any resources, even if the result set is empty, and would result\nin `{:error, :not_authorized}` if the user isn't authorized to read\nthe resources at all.","ref":"Janus.Authorization.html#any_authorized?/3"},{"type":"function","title":"Examples - Janus.Authorization.any_authorized?/3","doc":"iex> MyPolicy.any_authorized?(MyResource, :read, actor)\n    true\n\n    iex> MyPolicy.any_authorized?(MyResource, :delete, actor)\n    false","ref":"Janus.Authorization.html#any_authorized?/3-examples"},{"type":"callback","title":"Janus.Authorization.any_authorized?/3","doc":"","ref":"Janus.Authorization.html#c:any_authorized?/3"},{"type":"function","title":"Janus.Authorization.authorize/4","doc":"Authorizes a loaded resource.\n\nExpects to receive a struct, an action, and an actor or policy.\n\nReturns `{:ok, resource}` if authorized, otherwise `{:error, :not_authorized}`.","ref":"Janus.Authorization.html#authorize/4"},{"type":"function","title":"Options - Janus.Authorization.authorize/4","doc":"* `:load_associations` - Whether to load associations required by\n    policy authorization rules, defaults to `false` unless configured\n    on your policy module\n\n  * `:repo` - Ecto repository to use when loading required\n    associations if `:load_associations` is set to `true`, defaults to\n    `nil` unless configured on your policy module","ref":"Janus.Authorization.html#authorize/4-options"},{"type":"function","title":"Examples - Janus.Authorization.authorize/4","doc":"iex> MyPolicy.authorize(%MyResource{}, :read, actor) # accepts an actor\n    {:ok, %MyResource{}}\n\n    iex> MyPolicy.authorize(%MyResource{}, :read, policy) # or a policy\n    {:ok, %MyResource{}}\n\n    iex> MyPolicy.authorize(%MyResource{}, :delete, actor)\n    {:error, :not_authorized}","ref":"Janus.Authorization.html#authorize/4-examples"},{"type":"callback","title":"Janus.Authorization.authorize/4","doc":"","ref":"Janus.Authorization.html#c:authorize/4"},{"type":"function","title":"Janus.Authorization.scope/4","doc":"Create an `%Ecto.Query{}` that results in only authorized records.\n\nLike the `Ecto.Query` API, this function can accept a schema as the\nfirst argument or a query, in which case it will compose with that\nquery. If a query is passed, the appropriate schema will be derived\nfrom that query's source.\n\n    scope(MyResource, :read, user)\n\n    query = from(r in MyResource, where: r.inserted_at > from_ago(1, \"day\"))\n    scope(query, :read, user)\n\nIf the query specifies the source as a string, we cannot derive the\nschema. For example, this will not work:\n\n    # Raises an ArgumentError\n    query = from(r in \"my_resources\", where: r.inserted_at > from_ago(1, \"day\"))\n    scope(query, :read, user)","ref":"Janus.Authorization.html#scope/4"},{"type":"function","title":"Options - Janus.Authorization.scope/4","doc":"* `:preload_authorized` - Similar to `Ecto.Query.preload/3`, but\n  only preloads those associated records that are authorized. Note\n  that this requires Ecto v3.9.4 or later and a database that supports\n  lateral joins. See \"Preloading authorized associations\" for more\n  information.","ref":"Janus.Authorization.html#scope/4-options"},{"type":"function","title":"Preloading authorized associations - Janus.Authorization.scope/4","doc":"The `:preload_authorized` option can be used to preload associated\nrecords, but only those that are authorized for the given actor. An\nadditional query can be specified for each preloaded association that\nwill be run as if scoped to its parent row.\n\nThis can simplify certain queries dramatically. For instance, imagine\na user search interface that lists users along with their most recent\ncomment. Naughty comments can be hidden by moderators, but those\nhidden comments should still be visible if a moderator is searching.\nHere's how that might be accomplished:\n\n    iex> last_comment = from(Comment, order_by: [desc: :inserted_at], limit: 1)\n\n    iex> User\n    ...> |> search(search_params)\n    ...> |> MyPolicy.scope(:read, current_user,\n    ...>   preload_authorized: [comments: last_comment]\n    ...> )\n    ...> |> Repo.all()\n    [%User{comments: [%Comment{}]}, %User{comments: [%Comment{}]}, ...]\n\nSome things to note about this example:\n\n  * The `last_comment` query runs as if scoped to each user's\n    comments. This means that the `:limit` applies to each user's\n    comments, not the entire set of comments.\n\n  * The comment will be the last inserted comment that is authorized\n    to be read by the `current_user`. Moderators may be able to see\n    hidden comments, while normal users may not.\n\nIt is also possible to nest authorized preloads. For instance, you\ncould preload comments and their associated post.\n\n    MyPolicy.scope(User, :read, current_user,\n      preload_authorized: [comments: :post]\n    )\n\nThis would load all comments. You could incorporate the `last_comment`\nquery above by specifying it as the first element of a tuple, followed\nby the list of inner preloads:\n\n    MyPolicy.scope(User, :read, current_user,\n      preload_authorized: [comments: {last_comment, [:post]}]\n    )\n\nThis would load only the latest comment as well as its associated post\n(assuming it too is authorized to be read by `current_user`).","ref":"Janus.Authorization.html#scope/4-preloading-authorized-associations"},{"type":"function","title":"Examples - Janus.Authorization.scope/4","doc":"iex> MyPolicy.scope(MyResource, :read, actor)\n    %Ecto.Query{}\n\n    iex> MyPolicy.scope(MyResource, :read, actor) |> Repo.all()\n    [%MyResource{}, ...]\n\n    iex> MyResource\n    ...> |> MyPolicy.scope(:read, actor)\n    ...> |> order_by(inserted_at: :desc)\n    ...> |> limit(1)\n    ...> |> Repo.one()\n    %MyResource{}\n\n    iex> MyResource\n    ...> |> MyPolicy.scope(:read, actor,\n    ...>   preload_authorized: :other\n    ...> )\n    ...> |> Repo.all()\n    [%MyResource{other: %OtherResource{}}, ...]","ref":"Janus.Authorization.html#scope/4-examples"},{"type":"callback","title":"Janus.Authorization.scope/4","doc":"","ref":"Janus.Authorization.html#c:scope/4"},{"type":"type","title":"Janus.Authorization.filterable/0","doc":"","ref":"Janus.Authorization.html#t:filterable/0"},{"type":"behaviour","title":"Janus.Policy","doc":"Define composable authorization policies for actors in your system.\n\nA policy is a data structure created for an actor in your system that\ndefines the schemas that actor can access, the actions they can take,\nand any restrictions to the set of resources that can be accessed.\nThese policies are generally created implicitly for actors passed to\nfunctions defined by `Janus.Authorization`, but they can also be\ncreated with `c:build_policy/2`.","ref":"Janus.Policy.html"},{"type":"behaviour","title":"Creating a policy modules - Janus.Policy","doc":"While you can create a policy module with `use Janus.Policy`, you will\nusually invoke `use Janus` and implement `c:build_policy/2`:\n\n    defmodule MyApp.Policy do\n      use Janus\n\n      @impl true\n      def build_policy(policy, _actor) do\n        policy\n      end\n    end\n\nAn implementation for `c:build_policy/1` is injected into the policy\nmodule.\n\nPolicy modules can now be used to generate policy structs explicitly\n(though they will usually be created implicitly when calling functions\ndefined by `Janus.Authorization`).\n\n    iex> policy = MyApp.Policy.build_policy(:my_user)\n    %Janus.Policy{actor: :my_user, rules: %{...}}\n\n    iex> MyApp.SecondaryPolicy.build_policy(policy)\n    %Janus.Policy{actor: :my_user, rules: %{...}}","ref":"Janus.Policy.html#module-creating-a-policy-modules"},{"type":"behaviour","title":"Permissions with `allow` and `deny` - Janus.Policy","doc":"Permissions are primarily defined using `allow/4` and `deny/4`, which\nallows or denies an action on a resource if a set of conditions match.\nBoth functions take the same arguments and options. When permissions\nare being checked, multiple `allow` rules combine using logical-or,\nwith `deny` rules overriding `allow`.\n\nFor example, the following policy would allow a moderator to edit\ntheir own comments and any comments flagged for review, but not those\nmade by an admin.\n\n    @impl true\n    def build_policy(policy, %User{role: :moderator} = user) do\n      policy\n      |> allow(Comment, :update, where: [user: [id: user.id]])\n      |> allow(Comment, :update, where: [flagged_for_review: true])\n      |> deny(Comment, :update, where: [user: [role: :admin]])\n    end\n\nWhile set of keyword options passed to `allow` and `deny` are\nreminiscent of keyword-based Ecto queries, but since they are\nfunctions and not macros, there is no need to use the `^value` syntax\nused in Ecto. For example, the following would result in an error:\n\n    allow(policy, Comment, :update, where: [user: [id: ^user.id]])\n\n### `:where` and `:where_not` conditions\n\nThese conditions match if the associated fields are equal to each\nother. For instance, the moderation example above could also be\nwritten as:\n\n    @impl true\n    def build_policy(policy, %User{role: :moderator} = user) do\n      policy\n      |> allow(Comment, :update, where: [user_id: user.id])\n      |> allow(Comment, :update,\n        where: [flagged_for_review: true],\n        where_not: [user: [role: :admin]]\n      )\n    end\n\nMultiple conditions within the same `allow`/`deny` are combined with a\nlogical-and, so this might be translated to English as \"allow\nmoderators to edit comments they made or to edit comments flagged for\nreview that were not made by an admin\".\n\n### `:or_where` conditions\n\nYou can also use `:or_where` to combine with all previous conditions.\nFor instance, the two examples above could also be written as:\n\n    @impl true\n    def build_policy(policy, %User{role: :moderator} = user) do\n      policy\n      |> allow(Comment, :update,\n        where: [flagged_for_review: true],\n        where_not: [user: [role: :admin]],\n        or_where: [user_id: user.id]\n      )\n    end\n\nAn `:or_where` condition applies to all clauses before it. Using some\npseudocode for demonstration, the above would read:\n\n    # (flagged_for_review AND NOT user.role == :admin) OR user_id == user.id\n\nThese clauses could be reordered to have a different meaning:\n\n    policy\n    |> allow(Comment, :update,\n      where: [flagged_for_review: true],\n      or_where: [user_id: user.id],\n      where_not: [user: [role: :admin]]\n    )\n\n    # (flagged_for_review OR user_id == user.id) AND NOT user.role == :admin","ref":"Janus.Policy.html#module-permissions-with-allow-and-deny"},{"type":"behaviour","title":"Attribute checks with functions - Janus.Policy","doc":"When equality is not a sufficient check for an attribute, a function\ncan be supplied.\n\nFor instance, a `published_at` field might be used to schedule posts.\nUsers may only have permission to read posts where `published_at` is\nin the past, but we can only check for equality using the basic\nkeyword syntax presented above. In these cases, you can defer this\ncheck using an arity-3 function:\n\n    @impl true\n    def build_policy(policy, _actor) do\n      policy\n      |> allow(Comment, :read, where: [published_at: &in_the_past?/3])\n    end\n\n    def in_the_past?(:boolean, record, :published_at) do\n      if value = Map.get(record, :published_at) do\n        DateTime.compare(DateTime.utc_now(), value) == :gt\n      end\n    end\n\n    def in_the_past?(:dynamic, binding, :published_at) do\n      now = DateTime.utc_now()\n      Ecto.Query.dynamic(^now > as(^binding).published_at)\n    end\n\nAs seen in the example above, functions must define at least two\nclauses based on their first argument, `:boolean` or `:dynamic`, so\nthat they can handle both operations on a single record and operations\nthat should compose with an Ecto query.","ref":"Janus.Policy.html#module-attribute-checks-with-functions"},{"type":"behaviour","title":"Working with rulesets - Janus.Policy","doc":"Policies can also be defined by attaching rulesets created using\n`allow/3` and `deny/3`. Instead of taking a policy as a first argument,\nthese functions take a schema (or a ruleset).\n\nRulesets are specific to an individual schema and can be attached to\na policy using `attach/2`. For example:\n\n    @impl true\n    def build_policy(policy, actor) do\n      policy\n      |> attach(rules_for(Thread, actor))\n      |> attach(rules_for(Post, actor))\n    end\n\n    defp rules_for(Thread, %User{id: user_id}) do\n      Thread\n      |> allow(:read, where: [archived: false])\n      |> allow([:create, :update], where: [creator_id: user_id])\n    end\n\n    defp rules_for(Thread, nil) do\n      Thread\n      |> allow(:read, where: [archived: false, visibility: :public])\n    end\n\n    defp rules_for(Post, _actor) do\n      Post\n      |> allow(:read, where: [thread: allows(:read)])\n    end\n\nDepending on your specific needs, rulesets may allow you to organize\npolicies in a way that is easier to maintain. In the above example,\ndelegating to a private `rules_for/2` function that returns a ruleset\nallows us to pattern-match on a `nil` user where it matters and share\na ruleset where it doesn't.\n\nThis pattern has tradeoffs, however. You would need to ensure that the\npattern-matching for each schema is exhaustive, for instance, otherwise\na `FunctionClauseError` might be raised.","ref":"Janus.Policy.html#module-working-with-rulesets"},{"type":"behaviour","title":"Hooks - Janus.Policy","doc":"Functions can be registered as hooks that run prior to authorization\ncalls. See `attach_hook/4` for more information.","ref":"Janus.Policy.html#module-hooks"},{"type":"function","title":"Janus.Policy.allow/3","doc":"Creates or updates a ruleset for a schema to allow an action if matched\nby conditions.\n\nMust be attached to a policy using `attach/2`.\n\nSee \"Permissions with `allow` and `deny`\" for a description of conditions.","ref":"Janus.Policy.html#allow/3"},{"type":"function","title":"Examples - Janus.Policy.allow/3","doc":"thread_rules =\n      Thread\n      |> allow(:read)\n      |> allow(:create, where: [creator_id: user.id])\n\n    attach(policy, thread_rules)","ref":"Janus.Policy.html#allow/3-examples"},{"type":"function","title":"Janus.Policy.allow/4","doc":"Allows an action on the schema if matched by conditions.\n\nSee \"Permissions with `allow` and `deny`\" for a description of conditions.","ref":"Janus.Policy.html#allow/4"},{"type":"function","title":"Examples - Janus.Policy.allow/4","doc":"policy\n    |> allow(FirstResource, :read)\n    |> allow(SecondResource, :create, where: [creator: [id: user.id]])","ref":"Janus.Policy.html#allow/4-examples"},{"type":"function","title":"Janus.Policy.allows/1","doc":"Specifies that a condition should match if another action is allowed.\n\nIf used as the value for an association, the condition will match if\nthe action is allowed for the association.","ref":"Janus.Policy.html#allows/1"},{"type":"function","title":"Examples - Janus.Policy.allows/1","doc":"Allow users to edit any posts they can delete.\n\n    policy\n    |> allow(Post, :update, where: allows(:delete))\n    |> allow(Post, :delete, where: [user_id: user.id])\n\nDon't allow users to edit posts they can't read.\n\n    policy\n    |> allow(Post, :read, where: [archived: false])\n    |> allow(Post, :update, where: [user_id: user.id])\n    |> deny(Post, :update, where_not: allows(:read))","ref":"Janus.Policy.html#allows/1-examples"},{"type":"function","title":"Example with associations - Janus.Policy.allows/1","doc":"Let's say we have some posts with comments. Posts are visible unless\nthey are archived, and all comments of visible posts are also visible.\nTo start, we can duplicate the condition:\n\n    policy\n    |> allow(Post, :read, where: [archived: false])\n    |> allow(Comment, :read, where: [post: [archived: false]])\n\nIf we add additional clauses to the condition for posts, however, we\nwill have to duplicate them for comments. We can use `allows` instead:\n\n    policy\n    |> allow(Post, :read, where: [archived: false])\n    |> allow(Comment, :read, where: [post: allows(:read)])\n\nNow let's say we add a feature that allows for draft posts, which\nshould not be visible unless a `published_at` is set. We can modify\nonly the condition for `Post` and that change will propogate to\ncomments.\n\n    policy\n    |> allow(Post, :read, where: [archived: false], where_not: [published_at: nil])\n    |> allow(Comment, :read, where: [post: allows(:read)])","ref":"Janus.Policy.html#allows/1-example-with-associations"},{"type":"function","title":"Janus.Policy.attach/2","doc":"Attach a ruleset created using `allow/3` and `deny/3` to a policy.","ref":"Janus.Policy.html#attach/2"},{"type":"function","title":"Examples - Janus.Policy.attach/2","doc":"thread_rules =\n      Thread\n      |> allow(:read)\n      |> deny(:read, where: [scope: :private])\n\n    attach(policy, thread_rules)","ref":"Janus.Policy.html#attach/2-examples"},{"type":"function","title":"Janus.Policy.attach_hook/4","doc":"Attach a hook to the policy.\n\nExpects the following arguments:\n\n  * `policy` - the `%Janus.Policy{}` struct to attach to\n  * `name` - an atom identifying the hook\n  * `schema` (default `:all`) - an Ecto schema module identifying the\n    resource or query source that the hook should be applied to\n  * `fun` - the hook function, see \"Hooks\" below\n\nIf the given `name` is already present, an error will be raised. If\nyou wish to replace a hook, you can use `detach_hook/3` before\nre-attaching the hook. If you only wish to add a hook if it is hasn't\nalready been added, use `attach_new_hook/4` instead.\n\nHooks will be run in the order that they are attached.","ref":"Janus.Policy.html#attach_hook/4"},{"type":"function","title":"Hooks - Janus.Policy.attach_hook/4","doc":"Hooks are anonymous or captured functions that accept three arguments:\n\n  * `operation` - one of `:authorize` or `:scope`\n  * `object` - either a struct (for `:authorize`) or a queryable (for\n    `:scope`) that is being authorized\n  * `action` - the action being authorized\n\nHooks must return one of the following:\n\n  * `{:cont, object}` - additional hooks and authorization continue\n  * `:halt` - halt authorization, running no additional hooks and\n    returning `{:error, :not_authorized}` for an `:authorize`\n    operation and an empty query for a `:scope` operation","ref":"Janus.Policy.html#attach_hook/4-hooks"},{"type":"function","title":"Examples - Janus.Policy.attach_hook/4","doc":"When writing hooks, you must ensure that all possible arguments are\nhandled. This can be done using a \"catch-all\" clause. For example:\n\n    policy\n    |> attach_hook(:preload_user, fn\n      :authorize, %Post{} = resource, _action ->\n        {:cont, Repo.preload(resource, :user)}\n\n      _operation, object, _action ->\n        {:cont, object}\n    end)\n\n    policy\n    |> attach_hook(:preload_user, Post, fn\n      :authorize, resource, _action ->\n        {:cont, Repo.preload(resource, :user)}\n\n      _operation, object, _action ->\n        {:cont, object}\n    end)\n\nHooks can also be captured functions. For example:\n\n    policy\n    |> attach_hook(:preload_user, Post, &preload_user/3)\n\n    # elsewhere in your module\n\n    defp preload_user(:authorize, resource, _action) do\n      {:cont, Repo.preload(resource, :user)}\n    end\n\n    defp preload_user(:scope, query, _action) do\n      {:cont, from(query, preload: :user)}\n    end\n\nIf `:halt` is returned from a hook, no further hooks will be run and\nnothing will be authorized. This could be used to perform a check on\nbanned users, for example:\n\n    @impl true\n    def build_policy(policy, user) do\n      policy\n      |> attach_hook(:ensure_unbanned, fn _op, object, _action ->\n        if Accounts.banned?(user.id) do\n          :halt\n        else\n          {:cont, object}\n        end\n      end)\n    end\n\nThis may be required if policies are being cached, since the hook runs\nevery time the authorization call happens, instead of only once when\nthe policy is built.","ref":"Janus.Policy.html#attach_hook/4-examples"},{"type":"function","title":"Janus.Policy.attach_new_hook/4","doc":"Attach a new hook to the policy.\n\nLike `attach_hook/4`, except it only attaches the hook if the `name`\nisn't present for the given schema.","ref":"Janus.Policy.html#attach_new_hook/4"},{"type":"callback","title":"Janus.Policy.build_policy/1","doc":"Builds an authorization policy, delegating to `c:build_policy/2`.\n\nIf given a policy, calls `c:build_policy/2` with the policy and the\nactor associated with the policy. If given an actor, creates an empty\npolicy associated with that actor and passes it to `c:build_policy/2`.\n\nAn implementation for this callback is injected into modules invoking\neither `use Janus` or `use Janus.Policy`.","ref":"Janus.Policy.html#c:build_policy/1"},{"type":"callback","title":"Janus.Policy.build_policy/2","doc":"Builds an authorization policy containing rules for the given actor.\n\nSee `Janus.Policy` for API documentation on building policies.","ref":"Janus.Policy.html#c:build_policy/2"},{"type":"function","title":"Janus.Policy.deny/3","doc":"Creates or updates a ruleset for a schema to deny an action if matched\nby conditions.\n\nMust be attached to a policy using `attach/2`.\n\nSee \"Permissions with `allow` and `deny`\" for a description of conditions.","ref":"Janus.Policy.html#deny/3"},{"type":"function","title":"Examples - Janus.Policy.deny/3","doc":"thread_rules =\n      Thread\n      |> allow(:read)\n      |> deny(:read, where: [scope: :private])\n\n    attach(policy, thread_rules)","ref":"Janus.Policy.html#deny/3-examples"},{"type":"function","title":"Janus.Policy.deny/4","doc":"Denies an action on the schema if matched by conditions.\n\nSee \"Permissions with `allow` and `deny`\" for a description of conditions.","ref":"Janus.Policy.html#deny/4"},{"type":"function","title":"Examples - Janus.Policy.deny/4","doc":"policy\n    |> allow(FirstResource, :read)\n    |> deny(FirstResource, :read, where: [scope: :private])","ref":"Janus.Policy.html#deny/4-examples"},{"type":"function","title":"Janus.Policy.detach_hook/3","doc":"Detach a hook from the policy.","ref":"Janus.Policy.html#detach_hook/3"},{"type":"type","title":"Janus.Policy.hook/0","doc":"","ref":"Janus.Policy.html#t:hook/0"},{"type":"type","title":"Janus.Policy.ruleset/0","doc":"","ref":"Janus.Policy.html#t:ruleset/0"},{"type":"type","title":"Janus.Policy.t/0","doc":"","ref":"Janus.Policy.html#t:t/0"},{"type":"task","title":"mix janus.gen.policy","doc":"Creates a basic policy module.\n\n    $ mix janus.gen.policy [--module MODULE] [--path PATH]\n\nCreates the following file:\n\n  * `lib/app_name/policy.ex` - A Janus policy module containing\n    additional authorization helpers (see [Generated Policy Modules](generated_policy_modules.html)\n    for more info)","ref":"Mix.Tasks.Janus.Gen.Policy.html"},{"type":"task","title":"Options - mix janus.gen.policy","doc":"* `--module` - The name of the generated module, defaults to\n    `AppName.Policy`\n  * `--app` - The name of the application namespace, defaults to your\n    application name camelized, e.g. `AppName`\n  * `--path` - The path (including filename) for the generated module,\n    defaults to `lib/app_name/policy.ex`","ref":"Mix.Tasks.Janus.Gen.Policy.html#module-options"},{"type":"module","title":"Janus.Policy.Rule","doc":"Defines a rule for an individual schema and action.","ref":"Janus.Policy.Rule.html"},{"type":"type","title":"Janus.Policy.Rule.t/0","doc":"","ref":"Janus.Policy.Rule.html#t:t/0"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nThis project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).","ref":"changelog.html"},{"type":"extras","title":"v0.3.3 (2024-09-25) - Changelog","doc":"","ref":"changelog.html#v0-3-3-2024-09-25"},{"type":"extras","title":"Fixes - Changelog","doc":"* Support Ecto 3.12+.","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.3.2 (2023-02-14) - Changelog","doc":"","ref":"changelog.html#v0-3-2-2023-02-14"},{"type":"extras","title":"Fixes - Changelog","doc":"* [Janus.Policy] Add missing `@spec` for `allow/3` that was causing errors when running Dialyzer.","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.3.1 (2023-01-17) - Changelog","doc":"","ref":"changelog.html#v0-3-1-2023-01-17"},{"type":"extras","title":"Enhancements - Changelog","doc":"* [Janus.Policy] Add rulesets: `Janus.Policy.attach/2`, `Janus.Policy.allow/3`, `Janus.Policy.deny/3`.\n  * [mix janus.gen.policy] Simplify generated module and function docs.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Fixes - Changelog","doc":"* [Janus.Policy] Fix validation that was incorrectly raising when rules were defined for a schema module whose code hadn't yet been loaded.","ref":"changelog.html#fixes"},{"type":"extras","title":"v0.3.0 (2023-01-16) - Changelog","doc":"","ref":"changelog.html#v0-3-0-2023-01-16"},{"type":"extras","title":"Enhancements - Changelog","doc":"* [**Breaking** / Janus.Policy] Remove `:module` field from `%Janus.Policy{}` struct.\n  * [**Breaking** / Janus.Policy] Change schema/action argument order in `allow/4` and `deny/4` to be more consistent with the rest of Janus. See example below.\n  * [Janus.Policy] Add `c:Janus.Policy.build_policy/1` callback primarily to document its usage; an implementation was already injected into policy modules.\n  * [Janus.Policy] Add `:actor` field to `%Janus.Policy{}` struct.\n\n#### Example `allow/4` and `deny/4`\n\n```elixir\n# Old argument order\npolicy\n|> allow(:read, Thread, where: [...])\n|> allow(:create, Thread, where: [...])\n|> deny(:create, Thread, where: [...])\n\n# New argument order\npolicy\n|> allow(Thread, :read, where: [...])\n|> allow(Thread, :create, where: [...])\n|> deny(Thread, :create, where: [...])\n```","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.2.1 (2023-01-14) - Changelog","doc":"","ref":"changelog.html#v0-2-1-2023-01-14"},{"type":"extras","title":"Enhancements - Changelog","doc":"* [Janus.Policy] Raise if something other than a schema is passed to `allow/4` or `deny/4`.","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.2.0 (2023-01-05) - Changelog","doc":"","ref":"changelog.html#v0-2-0-2023-01-05"},{"type":"extras","title":"Enhancements - Changelog","doc":"* [**Breaking** / Janus.Policy] `policy_for` renamed to `build_policy`.\n  * [**Breaking** / Janus.Policy] `before_policy_for` removed.\n  * [**Breaking** / Janus.Authorization] `filter_authorized` renamed to `scope`.\n  * [**Breaking** / Janus.Authorization] `authorize` now returns `{:error, :not_authorized}` instead of `:error`.\n  * [Janus] `use Janus` accepts optional defaults for `Janus.Authorization.authorize/4` options.\n  * [Janus.Policy] Pre-authorization hooks with `Janus.Policy.attach_hook/4` and friends.\n  * [Janus.Authorization] `authorize/4` adds `:repo` and `:load_assocations` options.\n  * [mix janus.gen.policy] Overhaul generated policy helpers to more easily replace `Ecto.Repo` callbacks.","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.1.0 (2022-12-25) - Changelog","doc":"First release.","ref":"changelog.html#v0-1-0-2022-12-25"},{"type":"extras","title":"The Basics","doc":"# The Basics","ref":"basics.html"},{"type":"extras","title":"Setup - The Basics","doc":"{: .col-2}","ref":"basics.html#setup"},{"type":"extras","title":"Generator - The Basics","doc":"```sh\n$ mix janus.gen.policy\n* creating lib/my_app/policy.ex\n```\n\nGenerate a starting policy module for your application.\nFor more information, see the [Generated Policy Modules](generated_policy_modules.html) guide.","ref":"basics.html#generator"},{"type":"extras","title":"Policy module - The Basics","doc":"Your policy module is the interface used by the rest of your application.\nThis is usually the only place you should be referring to `Janus` directly.\n\n#### lib/my_app/policy.ex\n\n```elixir\ndefmodule MyApp.Policy do\n  use Janus\n\n  @impl true\n  def build_policy(policy, actor) do\n    policy\n    |> # authorization rules\n  end\nend\n```\n\nFor more information on defining policies, see the [Policy Definition Cheatsheet](policy_definition.html)","ref":"basics.html#policy-module"},{"type":"extras","title":"Core concepts - The Basics","doc":"{: .col-2}","ref":"basics.html#core-concepts"},{"type":"extras","title":"Schemas - The Basics","doc":"Schemas are modules that `use Ecto.Schema`.\n\n#### Used when defining policies\n\n```elixir\npolicy\n|> allow(Post, :edit, ...)\n#        ^^^^\n```\n\n#### Used when scoping a query\n\n```elixir\nMyApp.Policy.scope(Post, :edit, current_user)\n#                  ^^^^\n```\n\n#### Used when checking for permissions\n\n```elixir\nMyApp.Policy.any_authorized?(Post, :edit, current_user)\n#                            ^^^^\n```","ref":"basics.html#schemas"},{"type":"extras","title":"Resources - The Basics","doc":"Resources are loaded structs defined by one of your schemas.\n\n#### Used when authorizing an action\n\n```elixir\nMyApp.Policy.authorize(%Post{}, :edit, current_user)\n#                      ^^^^^^^\n```","ref":"basics.html#resources"},{"type":"extras","title":"Actors - The Basics","doc":"Actors are the users of your application.\nThey can be a `%User{}` struct, but they don't have to be.\nActors are converted to a policy using `build_policy/2`, so an actor can be anything that you want to use to differentiate between types of user.\nThey can even be a simple atom like `:normal_user` or `:admin_user`.\n\n#### In `build_policy/2`\n\n```elixir\ndef build_policy(policy, %User{}) do\n  #                      ^^^^^^^\nend\n```\n\n#### Used when calling any authorization function\n\n```elixir\nMyApp.Policy.authorize(%Post{}, :edit, current_user)\n#                                      ^^^^^^^^^^^^\n\nMyApp.Policy.scope(Post, :edit, current_user)\n#                               ^^^^^^^^^^^^\n```","ref":"basics.html#actors"},{"type":"extras","title":"Actions - The Basics","doc":"Actions are what actors _do to_ resources in your application.\nJanus doesn't care how you represent actions, but atoms usually do the trick.\n\n#### Used when defining policies\n\n```elixir\npolicy\n|> allow(Post, :edit, ...)\n#              ^^^^^\n```\n\n#### Used when calling any authorization function\n\n```elixir\nMyApp.Policy.authorize(%Post{}, :edit, current_user)\n#                               ^^^^^\n```\n\n#### Can be any term except a list\n\n```elixir\npolicy\n|> allow(Post, :edit, ...)\n|> allow(Post, \"edit\", ...)\n|> allow(Post, %Action{type: :edit}, ...)\n# lists are special-cased to allow multiple\n# actions to share conditions\n|> allow(Post, [:read, :edit],...)\n```","ref":"basics.html#actions"},{"type":"extras","title":"Defining rules - The Basics","doc":"{: .col-2}","ref":"basics.html#defining-rules"},{"type":"extras","title":"Overview - The Basics","doc":"Authorization rules are attached to policies in the `build_policy/2` callback.\n\n* `Janus.Policy.allow/4`\n* `Janus.Policy.deny/4`\n* `Janus.Policy.allows/1`","ref":"basics.html#overview"},{"type":"extras","title":"Grant permission for all resources of schema - The Basics","doc":"```elixir\npolicy\n|> allow(Post, :read)\n|> allow(Post, :edit)\n|> allow(Post, :archive)\n|> allow(Comment, :read)\n|> allow(Comment, :edit)\n```","ref":"basics.html#grant-permission-for-all-resources-of-schema"},{"type":"extras","title":"Using lists of actions - The Basics","doc":"```elixir\npolicy\n|> allow(Post, [:read, :edit, :archive], Post)\n|> allow(Comment, [:read, :edit], Comment)\n```","ref":"basics.html#using-lists-of-actions"},{"type":"extras","title":"Grant permission based on attributes - The Basics","doc":"```elixir\npolicy\n|> allow(Post, :read, where: [archived: false])\n\n# or define using :where_not\n|> allow(Post, :read, where_not: [archived: true])\n\n# or override a blanket permission using deny\n|> allow(Post, :read)\n|> deny(Post, :read, where: [archived: true])\n```","ref":"basics.html#grant-permission-based-on-attributes"},{"type":"extras","title":"Use `deny` to override a previous `allow` - The Basics","doc":"```elixir\npolicy\n|> allow(Post, :read)\n|> deny(Post, :read, where: [archived: true])\n```","ref":"basics.html#use-deny-to-override-a-previous-allow"},{"type":"extras","title":"Grant permission if the user is associated with the resource - The Basics","doc":"```elixir\ndef build_policy(policy, %User{role: :member} = user) do\n  policy\n  |> allow(Comment, :edit, where: [user_id: user.id])\nend\n```","ref":"basics.html#grant-permission-if-the-user-is-associated-with-the-resource"},{"type":"extras","title":"Grant permission based on association attributes - The Basics","doc":"```elixir\npolicy\n|> allow(Comment, :edit, where: [user: [role: :member]])\n```","ref":"basics.html#grant-permission-based-on-association-attributes"},{"type":"extras","title":"Use `allows` to delegate permission to an association - The Basics","doc":"```elixir\npolicy\n|> allow(Post, :read, where: [archived: false])\n|> allow(Comment, :read, where: [post: allows(:read)])\n```","ref":"basics.html#use-allows-to-delegate-permission-to-an-association"},{"type":"extras","title":"Multiple `allow` combines as a logical-or - The Basics","doc":"```elixir\n# This will always allow reading all posts\npolicy\n|> allow(Post, :read)\n|> allow(Post, :read, where: [archived: false]) # has no effect\n```","ref":"basics.html#multiple-allow-combines-as-a-logical-or"},{"type":"extras","title":"Hooks - The Basics","doc":"{: .col-2}","ref":"basics.html#hooks"},{"type":"extras","title":"Overview - The Basics","doc":"Hooks are attached to policies in the `build_policy/2` callback.\nThey are called prior to `authorize` or `scope` and can modify the resource/query or halt authorization altogether.\n\n* `Janus.Policy.attach_hook/4`\n* `Janus.Policy.attach_new_hook/4`\n* `Janus.Policy.detach_hook/3`","ref":"basics.html#overview"},{"type":"extras","title":"Run prior to authorizing any schema - The Basics","doc":"```elixir\npolicy\n|> attach_hook(:my_hook, fn\n  :authorize, resource, _action ->\n    {:cont, resource}\n\n  :scope, query, _action ->\n    {:cont, query}\nend)\n```","ref":"basics.html#run-prior-to-authorizing-any-schema"},{"type":"extras","title":"Run prior to authorizing a specific schema - The Basics","doc":"```elixir\npolicy\n|> attach_hook(:my_hook, Post, fn\n  :authorize, resource, _action ->\n    {:cont, resource}\n\n  :scope, query, _action ->\n    {:cont, query}\nend)\n```","ref":"basics.html#run-prior-to-authorizing-a-specific-schema"},{"type":"extras","title":"Used to preload fields - The Basics","doc":"```elixir\npolicy\n|> attach_hook(:preload_user, Post, fn\n  :authorize, post, _action ->\n    {:cont, Repo.preload(post, :user)}\n\n  :scope, query, _action ->\n    {:cont, query}\nend)\n```","ref":"basics.html#used-to-preload-fields"},{"type":"extras","title":"Remove attached hooks with `detach_hook/3` - The Basics","doc":"```elixir\npolicy\n|> attach_hook(:my_hook, &my_hook/3)\n|> detach_hook(:my_hook)\n```\n\n```elixir\npolicy\n|> attach_hook(:my_hook, Post, &my_hook/3)\n|> detach_hook(:my_hook, Post)\n```","ref":"basics.html#remove-attached-hooks-with-detach_hook-3"},{"type":"extras","title":"Attach hook if it's new using `attach_new_hook/4` - The Basics","doc":"```elixir\n# second call has no effect\npolicy\n|> attach_new_hook(:my_hook, &my_hook/3)\n|> attach_new_hook(:my_hook, &my_other_hook/3)\n```\n\n```elixir\n# second call has no effect\npolicy\n|> attach_new_hook(:my_hook, Post, &my_hook/3)\n|> attach_new_hook(:my_hook, Post, &my_other_hook/3)\n```\n\n```elixir\n# attaches second hook because :my_hook not added for Post\npolicy\n|> attach_new_hook(:my_hook, &my_hook/3)\n|> attach_new_hook(:my_hook, Post, &my_other_hook/3)\n```","ref":"basics.html#attach-hook-if-it-s-new-using-attach_new_hook-4"},{"type":"extras","title":"Run a late check before each authorization call - The Basics","doc":"```elixir\n@impl true\ndef build_policy(policy, user) do\n  policy\n  |> attach_hook(:ensure_unbanned, fn _, object, _ ->\n    if Accounts.banned?(user.id) do\n      :halt\n    else\n      {:cont, object}\n    end\n  end)\n  |> ...\nend\n```\n\nThis can be useful if policies are being cached. If the call to `Accounts.banned?(user.id)` occurred in the callback body instead, the policy could not react to any change in account status after it was built.","ref":"basics.html#run-a-late-check-before-each-authorization-call"},{"type":"extras","title":"Structuring your policies - The Basics","doc":"{: .col-2}","ref":"basics.html#structuring-your-policies"},{"type":"extras","title":"Pattern-match to give different permissions to different actors - The Basics","doc":"```elixir\ndef build_policy(policy, %User{role: :member}) do\n  # member permissions\nend\n\ndef build_policy(policy, %User{role: :moderator}) do\n  # moderator permissions\nend\n```","ref":"basics.html#pattern-match-to-give-different-permissions-to-different-actors"},{"type":"extras","title":"Delegate to context-specific policies - The Basics","doc":"```elixir\ndef build_policy(policy, actor) do\n  policy\n  |> CommunityForum.Policy.build_policy(actor)\n  |> Storefront.Policy.build_policy(actor)\nend\n```\n\nFor larger applications with well-defined boundaries, a policy can be constructed by threading it through multiple `build_policy` calls.","ref":"basics.html#delegate-to-context-specific-policies"},{"type":"extras","title":"Policy Usage","doc":"# Policy Usage","ref":"policy_usage.html"},{"type":"extras","title":"Authorization - Policy Usage","doc":"{: .col-2}","ref":"policy_usage.html#authorization"},{"type":"extras","title":"Using policies in a context module - Policy Usage","doc":"Policies should most often be used in context modules, since they provide the interface to actions and resources that the rest of your application uses.\n\n```elixir\ndefmodule MyApp.MessageBoard do\n  @moduledoc \"\"\"\n  Context module for the message board.\n  \"\"\"\n  import Ecto.Query\n\n  # imports authorize, any_authorized?, scope, etc.\n  import MyApp.Policy\n\n  alias MyApp.Repo\n\n  # ...\nend\n```","ref":"policy_usage.html#using-policies-in-a-context-module"},{"type":"extras","title":"Authorizing an action using `authorize/4` - Policy Usage","doc":"```elixir\ndef update_post(%Post{} = post, attrs \\\\ %{}, user_or_policy) do\n  case authorize(post, :edit, user_or_policy) do\n    {:ok, post} ->\n      post\n      |> Post.changeset(attrs)\n      |> Repo.update()\n\n    {:error, :not_authorized} ->\n      {:error, :not_authorized}\n  end\nend\n```\n\n```elixir\niex> update_post(authorized_post, %{}, some_user)\n{:ok, %Post{}}\n\niex> update_post(unauthorized_post, %{}, some_user)\n{:error, :not_authorized}\n```","ref":"policy_usage.html#authorizing-an-action-using-authorize-4"},{"type":"extras","title":"Fetching authorized resources - Policy Usage","doc":"```elixir\ndef authorized_posts(user_or_policy) do\n  if any_authorized?(Post, :read, user_or_policy) do\n    posts =\n      Post\n      |> scope(:read, user_or_policy)\n      |> Repo.all()\n\n    {:ok, posts}\n  else\n    {:error, :not_authorized}\n  end\nend\n```\n\nUse `any_authorized?/3` to differentiate between a result that is empty because there are no resources that match the policy conditions and a result that is empty because the user isn't authorized to view _any_ resources.","ref":"policy_usage.html#fetching-authorized-resources"},{"type":"extras","title":"Preloading authorized associations - Policy Usage","doc":"```elixir\nscope(Post, :read, user_or_policy,\n  preload_authorized: :comments\n)\n```\n\n```elixir\nscope(Post, :read, user_or_policy,\n  preload_authorized: [comments: :user]\n)\n```\n\nThe `:preload_authorized` option can be passed to preload only those associated resources that are authorized for the given action.","ref":"policy_usage.html#preloading-authorized-associations"},{"type":"extras","title":"Applying a query to preloads - Policy Usage","doc":"```elixir\nlatest_comment_query =\n  from Comment,\n    order_by: [desc: :inserted_at],\n    limit: 1\n\nscope(Post, :read, user_or_policy,\n  preload_authorized: [comments: latest_comment_query]\n)\n```\n\nA query can be applied to associated authorized resources.\nIt is scoped per-association, so it applies to comments of _each_ post instead of the comments of _all_ posts.\nThe above would return all `:read`-able posts preloaded with their latest `:read`-able comment.\n\nYou can still include nested preloads using a tuple:\n\n```elixir\nscope(Post, :read, user_or_policy,\n  preload_authorized: [comments: {latest_comment_query, [:user]}]\n)\n```","ref":"policy_usage.html#applying-a-query-to-preloads"},{"type":"extras","title":"Caching a policy - Policy Usage","doc":"{: .col-2}","ref":"policy_usage.html#caching-a-policy"},{"type":"extras","title":"Call `build_policy/1` to get the policy for a user - Policy Usage","doc":"```elixir\niex> policy = MyApp.Policy.build_policy(current_user)\n%Janus.Policy{...}\n```","ref":"policy_usage.html#call-build_policy-1-to-get-the-policy-for-a-user"},{"type":"extras","title":"Pass a policy anywhere you'd pass in an actor - Policy Usage","doc":"```elixir\niex> MyApp.Policy.authorize(post, :read, policy)\n{:ok, post}\n\niex> MyApp.Policy.scope(Post, :read, policy)\n%Ecto.Query{}\n```","ref":"policy_usage.html#pass-a-policy-anywhere-you-d-pass-in-an-actor"},{"type":"extras","title":"Cache a policy in a `Plug.Conn` - Policy Usage","doc":"#### Call in a plug\n\n```elixir\ndef assign_current_policy(conn) do\n  %{assigns: %{current_user: user}} = conn\n\n  conn\n  |> assign(:current_policy, MyApp.Policy.build_policy(user))\nend\n```\n\n#### Controller action\n\n```elixir\ndef index(conn, _params) do\n  %{assigns: %{current_policy: policy}} = conn\n\n  # Pass the policy to your context\n  case MessageBoard.authorized_posts(policy) do\n    {:ok, posts} ->\n      ...\n\n    {:error, :not_authorized} ->\n      ...\n  end\nend\n```","ref":"policy_usage.html#cache-a-policy-in-a-plug-conn"},{"type":"extras","title":"Generated Policy Modules","doc":"# Generated Policy Modules\n\n> #### Work in progress {: .warning}\n>\n> This guide is incomplete, though its contents may still be useful if you're getting started.\n\nJanus defines a small but flexible API that can be used to create a higher-level authorization interface for your application.\nTo support common conventions, like the use of Phoenix-style context modules, Janus provides a policy generator that defines helpers that are especially useful in those circumstances.\n\nThe goal of this guide is to explain the usage of, and reasoning behind, those helpers.","ref":"generated_policy_modules.html"},{"type":"extras","title":"Generating a policy module - Generated Policy Modules","doc":"The policy generator creates a policy module containing authorization functions that can be used by the rest of your application.\n\n```sh\n$ mix janus.gen.policy [--module Example.Policy] [--path example/path/policy.ex]\n```\n\nWhen run without arguments, it will generate a policy module called `YourApp.Policy` at `lib/your_app/policy.ex` (with `YourApp` replaced by your actual application namespace).","ref":"generated_policy_modules.html#generating-a-policy-module"},{"type":"extras","title":"Overview of helpers - Generated Policy Modules","doc":"Most of the generated helpers are designed to wrap common `Ecto.Repo` CRUD operations in a way that facilitates refactoring existing unauthorized calls to authorized ones with minimal necessary changes.\nThe helpers don't cover everything, but they cover a large portion of use-cases and provide a pattern for wrapping additional operations if needed.\n\nThe following helpers are included in newly-generated policy modules:\n\n* `authorized_fetch_by` - wraps `c:Ecto.Repo.get_by/3`, gets a resource using the given attributes and then authorizes for the given action/user\n\n* `authorized_fetch_all` - wraps `c:Ecto.Repo.all/2`, fetch a list of resources that are authorized for the given action/user\n\n* `authorized_insert` - wraps `c:Ecto.Repo.insert/2`, operates on a changeset, failing with a validation error if the inserted resource would not be authorized for the given action/user\n\n* `authorized_update` - wraps `c:Ecto.Repo.update/2`, operates on a changeset, failing with a validation error if the updated resource would not be authorized for the given action/user either before or after applying changes\n\n* `authorized_delete` - wraps `c:Ecto.Repo.delete/2`, deletes the given resource if it is authorized for the given action/user\n\n* `validate_authorized` - changeset validation that ensures the resource being changed is authorized, adding a validation error otherwise.\n\nWe'll go over each of these and how they might be used in the sections that follow.\n\n> #### Note on generated code {: .info}\n>\n> Remember that generated code is _your_ code!\n> It should be modified, replaced, and deleted as you see fit.\n>\n> Instead of including these helpers in `Janus`, they are generated as a starting point for your application.\n> They enable the usage patterns described below, but if they aren't useful to you, toss them and use whatever you see fit!","ref":"generated_policy_modules.html#overview-of-helpers"},{"type":"extras","title":"Authorized operations - Generated Policy Modules","doc":"The goal is to transition from unauthorized to authorized operations as smoothly as possible.\n`authorized_*` functions take the same arguments as their `Ecto.Repo` counterparts, except that they add additional keyword options related to authorization.\n\nLet's look at some examples.\n\n```elixir\niex> Repo.get_by(Post, id: 12345)\n%Post{}\n\niex> Policy.authorized_fetch_by(Post, [id: 12345], authorize: {:read, user})\n{:ok, %Post{}}\n# or\n{:error, :not_authorized}\n# or\n{:error, :not_found}\n```\n\nAll of the `authorized_*` functions take the `:authorize` keyword option, which should be either a tuple of `{action, actor}` as above, or `false` to skip authorization altogether.\nAn `ArgumentError` will be raised if `:authorize` is not present, but this can be changed if you'd prefer authorization to be opt-in instead of opt-out.\n(Opt-out, the default, is the recommended approach, though it may require more refactoring work up-front.)\n\nThere's another obvious difference in the example above: `authorized_fetch_by/3` returns an `:ok` or `:error` tuple instead of the resource or `nil`.\nReturning a tuple allows us to differentiate between a lookup failure and an auth failure.\n\nA similar approach is used in the next example:\n\n```elixir\niex> Repo.all(Post)\n[%Post{}, ...]\n\niex> Policy.authorized_fetch_all(Post, authorize: {:read, user})\n{:ok, [%Post{}, ...]}\n# or\n{:error, :not_authorized}\n```\n\nA tuple is also returned here to differentiate authorization failures and an empty result.\n\nAn Ecto query can also be passed as the first argument.\n\n```elixir\niex> query = from p in Post, where: p.inserted_at > ago(1, \"month\")\n\niex> Repo.all(query)\n[%Post{}, ...]\n\niex> Policy.authorized_fetch_all(query, authorize: {:read, user})\n{:ok, [%Post{}, ...]}\n# or\n{:error, :not_authorized}\n```","ref":"generated_policy_modules.html#authorized-operations"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}